
#ifndef OPERATIONMAKER_H
#define OPERATIONMAKER_H

// My report is flag used to control the reporting of errors in all the makers
// Errors can be reported to a message box, a trace file, both or neither
#ifndef MY_REPORT
#define MY_REPORT
#define REPORT_ERROR(...) operationmaker_report_error(__VA_ARGS__)
// Microsoft code for the message boxes called by REPORT_ERROR - comment out if not wanted.
// If not wanted must also change operationmaker_report_error code to not call the boxes
#include <windows.h>	
#endif

// Tracing of function calls for this maker can be eliminated by selecting appropriate definition for TRACE_CALL
#ifndef MY_TRACE
#define MY_TRACE
#ifndef TRACE_CALL	// select one
#define TRACE_CALL operationmaker_function_trace
//#define TRACE_CALL(...) 
#endif
#endif

#include "FeatureMaker.h"

// STIX defines functions to handle units and other useful utilities
#include <stp_schema.h>
#include <stix.h>

// ARM defines all the Application Objects in the ARM library (long list)
#include "ARM.h"

// File used by all the makers for tracing - must be defined in main program
extern FILE *pfLog;

// function calls are sent to a trace file or eliminated from code by choosing appropriate define for TRACE_FILE
void operationmaker_function_trace(const char * fmt, ...);

// Errors are reported to a message box, the function_trace file, both or neither
void operationmaker_report_error(const char * fmt, ...);

// Read the data in a file and make a catalog of its application objects
RoseDesign		*   read_and_catalog (const char *file_name);

// Utilities to find workpieces and features
Workpiece		*   find_workpiece_by_product_name (const char *product_name);
Workpiece		*   find_single_workpiece ();										// For files contain one workpiece only
stp_axis2_placement_3d	*   workpiece_placement (Workpiece *piece, double x, double y, double z, double i, double j, double k, double a, double b, double c);
Manufacturing_feature_IF *  find_default_feature_on_default_workpiece (RoseDesign *des);

/* Functions to make projects, workplans and selectives, workingsteps, operations and tool paths.
   These functions assume that another library will be used to make the required cutting tools (see for example ToolMaker.cpp)
   These functions also assume that other libraries will be used to make features and tolerances (see for example ToleranceMaker.cpp and FeatureMaker.cpp)
   */

// Make a new project
Project			*   new_project (RoseDesign *des, const char *project_name);						// Make project in file (only one allowed per file)

// Make a new workplan
Workplan		*   main_workplan (Project *project, const char *plan_name);						// make the new plan the main executable of the project
Workplan		*   new_workplan (Workplan *parent_plan, const char *plan_name);					// add new plan to end of given plan
Workplan		*   new_workplan (Workplan *parent_selective, const char *plan_name);					// add new plan as an option in a selective

// Define a Setup for a workplan, define a fixture for a setup and define placements for the workpiece in the fixture and the fixture on the machine
Setup			*   workplan_setup (Workplan *plan, StixMtrx M);
Setup			*   fixture_geometry (Setup *setup, Workpiece *model);
Setup			*   fixture_mount_on_machine (Setup *setup,  StixMtrx M);
Setup			*   workpiece_mount_on_fixture (Setup *setup,  StixMtrx M);
Setup			*   setup_security_plane (Setup* setup, StixMtrx M);

// Make a new selective
Selective		*   new_selective (Workplan *parent_plan, const char *selective_name);
Selective		*   new_selective (Selective *parent_selective, const char *selective_name);

// Insert and remove executabes into workplans and selectives
Workplan		*   insert_executable_after (Workplan *parent_workplan, Executable_IF *current, Executable_IF *insert);
Workplan		*   remove_executable (Workplan *parent_workplan, Executable_IF *remove);
Selective		*   insert_executable_after (Selective *plan, Executable_IF *current, Executable_IF *insert);
Selective		*   remove_executable (Selective *parent_selective, Executable_IF *remove);

// Make a workingstep (three flavors)
Machining_workingstep   *   new_workingstep (Workplan *parent_plan, const char *ws_name);					// Add to end of given workplan
Machining_workingstep   *   new_workingstep (Selective *parent_selective, const char *ws_name);					// Add to choices in a selective
bool			    check_workingstep (Machining_workingstep *ws);

// Make orphan processes for inserting
Machining_workingstep   *   orphan_workingstep (RoseDesign *des, const char *ws_name);						// Orphan until inserted
Workplan		*   oprhan_workplan (RoseDesign *des, const char *plan_name);						// Orphan until inserted
Selective		*   orphan_selective (RoseDesign *des, const char *selective_name);					// Orphan until inserted

// Additional information for workingsteps
Machining_workingstep	*   set_workingstep_security_plane (Machining_workingstep *ws, StixMtrx M);
Machining_workingstep	*   set_workingstep_toolpath_orientation (Machining_workingstep *ws, StixMtrx M);
Machining_workingstep   *   set_workingstep_final_feature (Machining_workingstep *ws, Manufacturing_feature_IF *feature);		// parametric description of geometry made by workingstep
Machining_workingstep   *   set_workingstep_process_feature (Machining_workingstep *ws, Manufacturing_feature_IF *feature);	// list of final part features impacted by this workingstep

// Define as_is, to_be and removal models for Workplan, Selective or Workingstep
Executable_IF		*   set_executable_as_is_workpiece (Executable_IF *exe, Workpiece *piece); 
Executable_IF		*   set_executable_to_be_workpiece (Executable_IF *exe, Workpiece *piece);
Executable_IF		*   set_executable_removal_workpiece (Workplan *plan, Workpiece *model);

// Operations applied to all types of executables
Executable_IF		*   enable_executable (Executable_IF *ws_or_plan_or_selective);
Executable_IF		*   disable_executable (Executable_IF *ws_or_plan_or_selective);

// Process technology data
Milling_technology	*   new_technology (RoseDesign *des, double feed, StixUnit feed_unit, double speed, StixUnit speed_unit);
Milling_machine_functions*  new_functions (RoseDesign *des, const char *coolant, const char *mist, const char *through_spindle, const char *chip_removal);
Milling_technology	*   find_or_make_technology (RoseDesign *des, double feed, StixUnit feed_unit, double speed, StixUnit speed_unit);
Milling_machine_functions * find_or_make_functions (RoseDesign *des, const char *coolant, const char *mist, const char*through_spindle, const char *chip_removal);


// Make a generic freeform operation (suitable when all the data is defined by toolpaths)
Machining_operation_IF	*   new_freeform_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs);

// Specific type of machining operations with parameters for key values
Freeform_rough_milling	*   new_freeform_rough_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs);
Freeform_finish_milling	*   new_freeform_finish_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs);
Plane_rough_milling	*   new_plane_rough_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double allowance, double axial_depth, StixUnit length_unit);
Plane_finish_milling	*   new_plane_finish_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double allowance, double axial_depth, StixUnit length_unit);
Bottom_and_side_rough_milling  *new_bottom_and_side_rough_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double side_allowance, double bottom_allowance, double radial_depth, double axial_depth, StixUnit length_unit);
Bottom_and_side_finish_milling *new_bottom_and_side_finish_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double side_allowance, double bottom_allowance, double radial_depth, double axial_depth, StixUnit length_unit);
Side_rough_milling	*   new_side_rough_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double side_allowance, double radial_depth, double axial_depth, StixUnit length_unit);
Side_finish_milling	*   new_side_finish_milling (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double side_allowance, double radial_depth, double axial_depth, StixUnit length_unit);

// drilling operations
Drilling		*   new_drilling_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double cutting_depth, double dwell_time_bottom, double feed_on_retract, double previous_diameter, 
				    StixUnit length_unit, StixUnit time_unit);
Multistep_drilling	*   new_multistep_drilling_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double cutting_depth, double depth_of_step, double first_depth, double retract_distance,
				    double dwell_time_bottom, double feed_on_retract, double dwell_time_step, double previous_diameter, 
				    StixUnit length_unit, StixUnit time_unit);
Tapping			*   new_tapping_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double cutting_depth, const char *compensation_chuck_required, double dwell_time_bottom, double feed_on_retract, double previous_diameter, 
				    StixUnit length_unit, StixUnit time_unit);
Reaming			*   new_reaming_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double cutting_depth, double dwell_time_bottom, double feed_on_retract, double previous_diameter,	const char *spindle_stop_at_bottom, 
				    double waiting_position_x, double waiting_position_y, double waiting_position_z, double depth_of_testcut,
				    StixUnit length_unit, StixUnit time_unit);
Boring			*   new_boring_operation (Machining_workingstep *ws, Milling_machine_cutting_tool_IF *tool, Milling_technology *tech, Milling_machine_functions *funs, 
				    double cutting_depth, double dwell_time_bottom, double feed_on_retract, double previous_diameter, const char *spindle_stop_at_bottom, 
				    double waiting_position_x, double waiting_position_y, double waiting_position_z, double depth_of_testcut,
				    StixUnit length_unit, StixUnit time_unit);

// Additional optional parameters for all operations
Machining_operation_IF	    *   set_operation_retract_distance (Machining_operation_IF *op, double retract_distance, StixUnit length_unit);
Machining_operation_IF	    *   set_operation_start_point (Machining_operation_IF *op, double x, double y, double z);

// Additional optional parameters for milling and drilling
Machining_operation_IF	    *   set_operation_overcut_length (Milling_type_operation_IF *op, double length, StixUnit length_unit);

// Approach and retract strategies for milling operations
Milling_type_operation_IF   *   set_plunge_toolaxis_approach (Milling_type_operation_IF *op);
//Milling_type_operation_IF   *   set_retract_plunge_toolaxis_strategy (Milling_type_operation_IF *op);
Milling_type_operation_IF   *   set_plunge_ramp_approach (Milling_type_operation_IF *op, double angle, StixUnit degrees_or_radians);
//Milling_type_operation_IF   *   set_retract_plunge_ramp_strategy (Milling_type_operation_IF *op, , double angle, StixUnit degrees_or_radians);
Milling_type_operation_IF   *   set_plunge_helix_approach (Milling_type_operation_IF *op, double angle, double radius, StixUnit degrees_or_radians, StixUnit length_unit);
//Milling_type_operation_IF   *   set_retract_plunge_helix_strategy (Milling_type_operation_IF *op, double angle, double radius, StixUnit degrees_or_radians, StixUnit length_unit);
Milling_type_operation_IF   *   set_plunge_zigzag_approach (Milling_type_operation_IF *op, double angle, double width, StixUnit degrees_or_radians, StixUnit length_unit);
//Milling_type_operation_IF   *   set_retract_plunge_zigzag_strategy (Milling_type_operation_IF *op, double angle, double width, StixUnit degrees_or_radians, StixUnit length_unit);
Milling_type_operation_IF   *   set_air_angle_approach (Milling_type_operation_IF *op, double angle, double travel_length, StixUnit degrees_or_radians, StixUnit length_unit);
Milling_type_operation_IF   *   set_air_angle_retract (Milling_type_operation_IF *op, double angle, double travel_length, StixUnit degrees_or_radians, StixUnit length_unit);
Milling_type_operation_IF   *   set_air_tangent_approach (Milling_type_operation_IF *op, double radius, StixUnit length_unit);
Milling_type_operation_IF   *   set_air_tangent_retract (Milling_type_operation_IF *op, double radius, StixUnit length_unit);

// Approaches and retract that use explict tool paths
Milling_type_operation_IF   *   set_along_path_approach (Milling_type_operation_IF *op, Toolpath_IF *path);
Milling_type_operation_IF   *   add_path_to_approach (Milling_type_operation_IF *op, Toolpath_IF *path);
Milling_type_operation_IF   *   set_along_path_retract (Milling_type_operation_IF *op, Toolpath_IF *path);
Milling_type_operation_IF   *   add_path_to_retract (Milling_type_operation_IF *op, Toolpath_IF *path);

// For five axis machining, optional description of the tool orientation at the start and end of each operation 
Milling_type_operation_IF   *   set_operation_start_orientation (Milling_type_operation_IF *op, double i, double j, double k);
Milling_type_operation_IF   *   set_operation_end_orientation (Milling_type_operation_IF *op, double i, double j, double k);

// Operation strategies for milling
Two5d_milling_operation_IF  *   set_contour_spiral_strategy (Two5d_milling_operation_IF * op, const char *cutmode, const char *rot_direction, double overlap_ratio);
Two5d_milling_operation_IF  *   set_contour_parallel_strategy (Two5d_milling_operation_IF * op, const char *cutmode, const char *rot_direction, double overlap_ratio);
Two5d_milling_operation_IF  *   set_contour_bidirectional_strategy (Two5d_milling_operation_IF * op, const char *spiral_cutmode, 
				double i, double j, double k, const char *rot_direction, const char * stepover_direction, double overlap_ratio);
Two5d_milling_operation_IF  *   set_bidirectional_contourstrategy (Two5d_milling_operation_IF * op,  const char *spiral_cutmode,  
				double i, double j, double k, const char *rot_direction, const char * stepover_direction, double overlap_ratio);
Two5d_milling_operation_IF  *   set_unidirectional_strategy (Two5d_milling_operation_IF * op,  const char *cutmode, double i, double j, double k, double overlap_ratio);
Two5d_milling_operation_IF  *   set_bidirectional_strategy (Two5d_milling_operation_IF * op,  
				double i, double j, double k, const char * stepover_direction, const char *connection_strategy, double overlap_ratio);

// simple strategies
Two5d_milling_operation_IF  *   set_center_strategy (Two5d_milling_operation_IF * op);
Two5d_milling_operation_IF  *   set_explicit_strategy (Two5d_milling_operation_IF * op);

// Common parameter of all strategies that can be set in case want to override multiple depth passes
Two5d_milling_operation_IF *	set_strategy_disallow_multiple_passes (Two5d_milling_operation_IF * op);


// Create a toolpath segment for an operation
void				set_toolpath_units (Machining_operation_IF *op, StixUnit length_unit, StixUnit angle_unit);
Cutter_location_trajectory *	new_toolpath (Machining_operation_IF *op, const char* type, const char *name);
Cutter_location_trajectory *	new_toolpath_rapid (Machining_operation_IF *op, const char* type, const char *name);
Cutter_contact_trajectory  *	new_toolpath_contact (Machining_operation_IF *op, const char* type, const char *name);
Cutter_contact_trajectory  *	new_toolpath_contact_rapid (Machining_operation_IF *op, const char* type, const char *name);

// Optional attributes for toolpath
// Set functions for a toolpath when different from those of the operation
Toolpath_IF		*   set_toolpath_functions (Toolpath_IF *path, Milling_machine_functions *funs);
// Set technology for a toolpath when different from those of the operation
Toolpath_IF		*   set_toolpath_technology (Toolpath_IF *path, Milling_technology *tech);
// Max deviation for toolpaths
Toolpath_IF		*   set_toolpath_maximum_deviation (Toolpath_IF *path, double value, StixUnit length_unit);
// Priority for toolpath (can it be deleted (EN_SUGGESTED the OperationMaker default) or is it required (EN_REQUIRED))
Toolpath_IF		*   set_toolpath_priority (Toolpath_IF *path, const char * priority);

// Add curves to the toolpath
stp_cartesian_point	*   start_tool_tip_curve (Toolpath_IF *path, double x, double y, double z);
stp_cartesian_point	*   start_tool_axis_curve (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   start_surface_normal_curve (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   start_reference_direction_curve (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   start_cross_section_7d_curve (Toolpath_IF *path, double RC_max, double AC_max, double X_offset, double Y_offset, double RD_max, double RD_avg, double RD_offset);
bool			    check_toolpath_curves (Toolpath_IF *path);

// Many paths start with the last point of the previous path
stp_cartesian_point	*   start_tool_tip_curve (Toolpath_IF *path, stp_cartesian_point *first);
stp_cartesian_point	*   start_tool_axis_curve (Toolpath_IF *path, stp_cartesian_point *first);
stp_cartesian_point	*   start_surface_normal_curve (Toolpath_IF *path, stp_cartesian_point *first);
stp_cartesian_point	*   start_reference_direction_curve (Toolpath_IF *path, stp_cartesian_point *first);
stp_cartesian_point	*   start_cross_section_7d_curve (Toolpath_IF *path, stp_cartesian_point *first);

// Functions for extending the tool tip curve
stp_cartesian_point	*   goto_xyz (Toolpath_IF *path, double x, double y, double z);
stp_via_arc_point	*   goto_arc (Toolpath_IF *path, double x, double y, double z);		// via point to create an arc from pt[-1] and pt[+1]
stp_via_helix_point	*   goto_helix (Toolpath_IF *path, double x, double y, double z);	// via point to create an arc from pt[-1] and pt[+1]

// Other types of curves
stp_cartesian_point	*   goto_axis (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   goto_normal (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   goto_direction (Toolpath_IF *path, double i, double j, double k);
stp_cartesian_point	*   goto_cross_section (Toolpath_IF *path, double RC_max, double AC_max, double X_offset, double Y_offset, double RD_max, double RD_avg, double RD_offset);

// All curves must have the same basis (number of points) use these functions when a curve value does not change or is repeated at the new point
stp_cartesian_point	*   goto_xyz (Toolpath_IF *path, stp_cartesian_point *repeat);
stp_cartesian_point	*   goto_axis (Toolpath_IF *path, stp_cartesian_point *repeat);
stp_cartesian_point	*   goto_normal (Toolpath_IF *path, stp_cartesian_point *repeat);
stp_cartesian_point	*   goto_direction (Toolpath_IF *path, stp_cartesian_point *repeat);
stp_cartesian_point	*   goto_cross_section (Toolpath_IF *path, stp_cartesian_point *repeat);

// Many types of geometry are allowed in a stp_bounded_curve - first round STEP-NC implementations used trimmed arcs but representation was very verbose 
stp_trimmed_curve	*   add_xy_arc(stp_bounded_curve *curve, double new_x, double new_y, double new_z, double cx, double cy, double cz, double radius, bool ccw);
stp_bounded_curve	*   add_yz_arc(stp_bounded_curve *curve, double new_x, double new_y, double new_z, double cx, double cy, double cz, double radius, bool ccw);
stp_bounded_curve	*   add_zx_arc(stp_bounded_curve *curve, double new_x, double new_y, double new_z, double cx, double cy, double cz, double radius, bool ccw);
stp_bounded_curve	*   add_general_arc(double new_x, double new_y, double new_z,  double cx, double cy, double cz, double ci, double cj, double ck,
				    double ca, double cb, double cc, double radius, bool ccw, const char * label);

stp_circle		*   add_xy_circle(stp_bounded_curve *curve, double cx, double cy, double cz, double radius, bool ccw, const char * label);
stp_bounded_curve	*   add_yz_circle(stp_bounded_curve *curve, double cx, double cy, double cz, double radius, bool ccw, const char * label);
stp_bounded_curve	*   add_zx_circle(stp_bounded_curve *curve, double cx, double cy, double cz, double radius, bool ccw, const char * label);
stp_bounded_curve	*   add_general_circle(stp_bounded_curve *curve, double cx, double cy, double cz, double ci, double cj, double ck, double ca, double cb, double cc, double radius, bool ccw, const char * label); 

#endif